function GSM_S_m(block)

% Simulationsmodell fr das Gesamtsystem der Propellermaschine
%
% -------------------------------------------------------------------------
%
% Beschreibung: Simulationsmodell des Gesamtsystems der Propellermaschine
%               Regelungstechnik Laborbung fr Technische Informatik
%               Aufgabe 2.2
%
% -------------------------------------------------------------------------
%
% Parameter: 
%           p(1)... i_GSM    Anfangswert i_GSM
%           p(2)... phi_GSMP Anfangswert phi_GSMP
%           p(3)... w_GSMP   Anfangswert w_GSMP
%           p(4)... w_P      Anfangswert w_P
%           p(5)... L_GSM
%           p(6)... R_GSM
%           p(7)... k_GSM
%           p(8)... J_GSM
%           p(9)... d_cGSM
%           p(10)... d_vGSM
%           p(11)... J_P
%           p(12)... d_cP
%           p(13)... d_vP
%           p(14)... d_qP
%           p(15)... c_GSMP
%           p(16)... d_GSMP
%
% -------------------------------------------------------------------------
% Abtastzeit (sample time): zeitkontinuierlich (continuous)
% -------------------------------------------------------------------------


    %%
    %% The setup method is used to set up the basic attributes of the
    %% S-function such as ports, parameters, etc. Do not add any other
    %% calls to the main body of the function.
    %%
    setup(block);

%endfunction

%% Function: setup ===================================================
%% Abstract:
%%   Set up the basic characteristics of the S-function block such as:
%%   - Input ports
%%   - Output ports
%%   - Dialog parameters
%%   - Options
%%
%%   Required         : Yes
%%   C-Mex counterpart: mdlInitializeSizes
%%
function setup(block)

    % Register number of ports
    block.NumInputPorts  = 2;
    block.NumOutputPorts = 2;

    % Anzahl der zeitkontinuierlichen Zustaende
    block.NumContStates = 4;

    % Anzahl der Parameter
    block.NumDialogPrms = 16; % 12 Konstanten + 4 Anfangswerte fr Zustnde

    % Override input port properties
    block.InputPort(1).Dimensions        = 1;
    block.InputPort(1).SamplingMode = 'Sample';
    block.InputPort(1).DirectFeedthrough = false;
    block.InputPort(2).Dimensions        = 1;
    block.InputPort(2).SamplingMode = 'Sample';
    block.InputPort(2).DirectFeedthrough = false;

    % Override output port properties
    block.OutputPort(1).Dimensions       = 4;
    block.OutputPort(1).SamplingMode = 'Sample';
    block.OutputPort(2).Dimensions       = 2;
    block.OutputPort(2).SamplingMode = 'Sample';


    % Register sample times
    %  [0 offset]            : Continuous sample time
    %  [positive_num offset] : Discrete sample time
    %
    %  [-1, 0]               : Inherited sample time
    %  [-2, 0]               : Variable sample time
    block.SampleTimes = [0 0];

    % Specify the block simStateCompliance. The allowed values are:
    %    'UnknownSimState', < The default setting; warn and assume DefaultSimState
    %    'DefaultSimState', < Same sim state as a built-in block
    %    'HasNoSimState',   < No sim state
    %    'CustomSimState',  < Has GetSimState and SetSimState methods
    %    'DisallowSimState' < Error out when saving or restoring the model sim state
    %block.SimStateCompliance = 'DefaultSimState';

    %% -----------------------------------------------------------------
    %% The MATLAB S-function uses an internal registry for all
    %% block methods. You should register all relevant methods
    %% (optional and required) as illustrated below. You may choose
    %% any suitable name for the methods and implement these methods
    %% as local functions within the same file. See comments
    %% provided for each function for more information.
    %% -----------------------------------------------------------------

    block.RegBlockMethod('InitializeConditions', @InitializeConditions);
    block.RegBlockMethod('Outputs', @Outputs);     % Required
    block.RegBlockMethod('Derivatives', @Derivatives);
    block.RegBlockMethod('Terminate', @Terminate); % Required

%end setup



%%
%% InitializeConditions:
%%   Functionality    : Called at the start of simulation and if it is 
%%                      present in an enabled subsystem configured to reset 
%%                      states, it will be called when the enabled subsystem
%%                      restarts execution to reset the states.
%%   Required         : No
%%   C-MEX counterpart: mdlInitializeConditions
%%
function InitializeConditions(block)
    %Anfangszuzstnde aus Parametern auslesen
    i_GSM0    = block.DialogPrm(1).Data;
    phi_GSMP0 = block.DialogPrm(2).Data;
    w_GSMP0   = block.DialogPrm(3).Data;
    w_P0      = block.DialogPrm(4).Data;

    % Eingabe der Anfangsbedingungen
    x0(1) = i_GSM0;
    x0(2) = phi_GSMP0;
    x0(3) = w_GSMP0;
    x0(4) = w_P0;

    % Schreiben auf Objekt block (NICHT VERAENDERN)
    block.ContStates.Data = x0;
%end InitializeConditions

%%
%% Outputs:
%%   Functionality    : Called to generate block outputs in
%%                      simulation step
%%   Required         : Yes
%%   C-MEX counterpart: mdlOutputs
%%
function Outputs(block)
    L_GSM  = block.DialogPrm(5).Data;
    R_GSM  = block.DialogPrm(6).Data;
    k_GSM  = block.DialogPrm(7).Data;
    J_GSM  = block.DialogPrm(8).Data;
    d_cGSM = block.DialogPrm(9).Data;
    d_vGSM = block.DialogPrm(10).Data;
    J_P    = block.DialogPrm(11).Data;
    d_cP   = block.DialogPrm(12).Data;
    d_vP   = block.DialogPrm(13).Data;
    d_qP   = block.DialogPrm(14).Data;
    c_GSMP = block.DialogPrm(15).Data;
    d_GSMP = block.DialogPrm(16).Data;
    
    % Shortcut fuer den Zustand
    x = block.ContStates.Data;
    i_GSM    = x(1);
    phi_GSMP = x(2);
    w_GSM    = x(3);
    w_P      = x(4);
    
    % Berechnung der Ausgaenge
    y1(1) = i_GSM;
    y1(2) = phi_GSMP;
    y1(3) = w_GSM;
    y1(4) = w_P;
    
    M_GSM =  k_GSM*i_GSM;
    M_KOPP = (w_GSM-w_P)*d_GSMP + phi_GSMP*c_GSMP;
    
    y2(1) = M_GSM;
    y2(2) = M_KOPP;
    
    % Schreiben auf Objekt block
    block.OutputPort(1).Data = y1;
    block.OutputPort(2).Data = y2;
    
%end Outputs

%%
%% Derivatives:
%%   Functionality    : Called to update derivatives of
%%                      continuous states during simulation step
%%   Required         : No
%%   C-MEX counterpart: mdlDerivatives
%%
function Derivatives(block)
    L_GSM  = block.DialogPrm(5).Data;
    R_GSM  = block.DialogPrm(6).Data;
    k_GSM  = block.DialogPrm(7).Data;
    J_GSM  = block.DialogPrm(8).Data;
    d_cGSM = block.DialogPrm(9).Data;
    d_vGSM = block.DialogPrm(10).Data;
    J_P    = block.DialogPrm(11).Data;
    d_cP   = block.DialogPrm(12).Data;
    d_vP   = block.DialogPrm(13).Data;
    d_qP   = block.DialogPrm(14).Data;
    c_GSMP = block.DialogPrm(15).Data;
    d_GSMP = block.DialogPrm(16).Data;
    
    % Shortcut fuer die Eingaenge
    u = block.InputPort(1).Data;
    u_GSM = u(1);
    
    d = block.InputPort(2).Data;
    M_ext = d(1);
  
    % Shortcut fuer den Zustand
    x = block.ContStates.Data;
    i_GSM    = x(1);
    phi_GSMP = x(2);
    w_GSM   = x(3);
    w_P      = x(4);
    
    % Gleichungen fr x Punkt:
    di_GSM = 1/L_GSM*(u_GSM-R_GSM*i_GSM-k_GSM*w_GSM);
    w_GSMP = w_GSM-w_P;
    dw_GSM = 1/J_GSM*(k_GSM*i_GSM-d_cGSM-d_vGSM*w_GSM+(w_P-w_GSM)*d_GSMP-phi_GSMP*c_GSMP);
    dw_P   = 1/J_P*((w_GSM-w_P)*d_GSMP+phi_GSMP*c_GSMP-d_cP-d_vP*w_P-d_qP*w_P^2-M_ext);

    dx = [di_GSM,w_GSMP,dw_GSM,dw_P];
    
    % Schreiben auf Objekt block
    block.Derivatives.Data = dx;
%end Derivatives

%%
%% Terminate:
%%   Functionality    : Called at the end of simulation for cleanup
%%   Required         : Yes
%%   C-MEX counterpart: mdlTerminate
%%
function Terminate(block)

%end Terminate

